-module(futam).
-compile(export_all).
-author('hanak@iit.bme.hu, modified by: kapolnai@iit.bme.hu').
-vsn('$LastChangedDate: 2011-12-01 11:29:32 +0100 (Thu, 01 Dec 2011) $$').

%%% Első változat: futam és maradék előállítása két függvénnyel

%% @type pred() = fun(elem(), elem()) -> bool().
%% @type elem() = any().
%% @spec naiv_futamok(Pred::pred(), List::[elem()]) -> Lists:[[elem()]].
%% Lists a List szomszédos elemeiből álló, Pred-et kielégítő futamok listája.
naiv_futamok(_P, []) ->
    [];
naiv_futamok(P, List) ->
    Fs = elso_futam(P, List),
    Ms = maradek(P, List),
    [Fs|naiv_futamok(P, Ms)].

%% @spec elso_futam(P::pred(), List::[elem()]) -> Fs::[elem()].
%% Fs a List P-t kielégítő első (prefix) futama.
elso_futam(_P, [X]) ->
    [X];
elso_futam(P, [X|[Y|_]=Ys]) ->
    case P(X, Y) of
        true  -> [X|elso_futam(P, Ys)];
        false -> [X]
    end.

%% @spec maradek(P::pred(), List::[elem()]) -> Ms::[elem()].
%% Ms a List P-t kielégítő első futam utáni maradéka.
maradek(_P, [_X]) ->
    [];
maradek(P, [X|[Y|_]=Ys]) ->
    case P(X, Y) of
        true  -> maradek(P, Ys);
        false -> Ys
    end.

%%% Második változat: futam és maradék előállítása egy függvénnyel

%% elso_futam(P, L) az L P-t kielégítő első (prefix) futama.
%% maradek(P, L) az L P-t kielégítő első futam utáni maradéka.
%% @spec futam_maradek(P::pred(), L::[elem()]) -> {Fs::[elem()],Ms::[elem()]}.
%% Fs =:= elso_futam(P, L), Ms =:= maradek(P, L).
futam_maradek(_P, [X]) ->
    {[X], []};
futam_maradek(P, [X|Ys=[Y|_]]) ->
    case P(X, Y) of
        true ->
            {Fs, Ms} = futam_maradek(P, Ys),
            {[X|Fs], Ms};
        false ->
            {[X], Ys}
    end.

%% futamok(P, L) =:= naiv_futamok(P, L).
futamok(_P, []) ->
    [];
futamok(P, L) ->
    {Fs, Ms} = futam_maradek(P, L),
    [Fs|futamok(P, Ms)].

%%% Példa
test() ->
    L = [1,3,9,5,7,2,5,9,1,6,0,0,3,5,6,2],
    L2 = [1,9,19,3,4,24,34,4,11,45,66,13,45,66,99],
    [
     naiv_futamok(fun erlang:'<'/2, L),
     futam_maradek(fun erlang:'<'/2, L),
     futamok(fun erlang:'<'/2, L),
     futamok(fun erlang:'>='/2, L),
     futamok(fun(A,B) -> 2*A>B end, L),
     elso_futam(fun erlang:'=<'/2, L2), %%% [1,9,19]
     maradek(fun erlang:'=<'/2, L2), %%% [3,4,24,34,4,11,45,66,13,45,66,99].
     naiv_futamok(fun erlang:'=<'/2, L2), %%% [[1,9,19], [3,4,24,34], [4,11,45,66], [13,45,66,99]].
     naiv_futamok(fun(A,B) -> A=<B end, [299,1]), %%% [[299], [1]].
     naiv_futamok(fun(A,B) -> A=<B end, [299]), %%% [[299]].
     naiv_futamok(fun(A,B) -> A=<B end, []), %%% [].
     difek([1,3,5,7,7,5,3,1,1,1,1,2])
    ].


%% @spec difek(Xs::[number()]) -> Dss::[[number()]].
%% A Dss lista az Xs lista számtani sorozatot alkotó részlistáinak listája.
difek([X1,X2|_]=L) ->
    {Fs,Ms} = futam_maradek(fun(A, B) -> B-A=:=X2-X1 end, L),
    [Fs|difek(Ms)];
difek([_]=L) ->
    [L];
difek([]) ->
    [].
    
